//OSC
import oscP5.*;
import netP5.*;
  
OscP5 oscP5;
OscP5 oscP6;
NetAddress myBroadcastLocation; 

//syph stuff:
import codeanticode.syphon.*;
PGraphics canvas;
SyphonServer server;

// kinect
import SimpleOpenNI.*; 
SimpleOpenNI  kinect;
PImage liveMap;

// ------------ SETUP -----------
// setting the primary limits in cm.
// Everything between 1,5m and 3m will form the mask
int front = 1500;           
int back = 3000;

// osc ports
int portIn = 12000;
int portOut = 1234;
int mode = 0;
int unused = 0;

PFont mono;
// -----------------------


void setup()
{  
  size(640, 480, P2D);
  frameRate(30);
  mono = loadFont("Monaco-12.vlw");
  
  //OSC
  myBroadcastLocation = new NetAddress("127.0.0.1",1234);
  oscP5 = new OscP5(this,12000);
  oscP5.plug(this,"trap","/trap");
  oscP6 = new OscP5(this,12001);
  oscP6.plug(this,"mode","/mode");

  //KINECT
  
  // make and check a kinect connection
  
  kinect = new SimpleOpenNI(this);
  
  if (kinect.isInit() == false)
  {
    println("Can't init SimpleOpenNI, maybe the camera is not connected!"); 
    exit();
    return;
  }
    
  // set mirror state of your kinect (depends on your setting and projector setup)
  kinect.setMirror(false);
  // enable depthMap generation 
  kinect.enableDepth();
  // enable ir generation
  kinect.enableIR();
 
  // load a placeholder Image (will be overwritten and become our liveMap mask image) 
  liveMap = loadImage("try_640_480.jpg");

  // make a syphon server
  server = new SyphonServer(this, "mask");

}

public void trap(int theA, int theB) {
//println("### plug event method. received a message /trap.");
//println(" 2 ints received: "+theA+", "+theB);  
  front = theA;
  back = theB;
 // mode = theC;
}
 
 public void mode(int theA, int theB) {
//println("### plug event method. received a message /mode.");
//println(" 2 ints received: "+theA+", "+theB);  
mode = theA;
unused = theB;

}



void fpsSend(){
  OscMessage myOscMessage = new OscMessage("/isadora/1");
  myOscMessage.add(round(frameRate));
  oscP5.send(myOscMessage, myBroadcastLocation);
}


void draw()
{
  background(color(0,0,0));  


fpsSend();

  
  // draw & syphon the image
  
  switch(mode) {
  case 0: 
  //  println("trap");  // Does not execute
      kinect.update();
  int[] depthValues = kinect.depthMap();
  // init liveMap Image
  liveMap.width = 640;
  liveMap.height = 480;
  liveMap.loadPixels();
  // Now create the live Mask:
  // run through the lines and cols of depthImage
  // and set the pixels of liveMap to black or white
  // according to the distance of the appropriate
  // pixel in depthImage:
  for (int y=0; y<480; y++) {
    for (int x=0; x<640; x++) {
      int i= x+(y*640);
      int currentDepthValue = depthValues[i];
      if (currentDepthValue>front&&currentDepthValue<back) {
        liveMap.pixels[i] = color(255,255,255);  
      } else {
        liveMap.pixels[i] = color(0,0,0);            
      }
    }  
  }
  // overwrite the pixels on liveMap 
  liveMap.updatePixels();
      image(liveMap,0,0);
  server.sendImage(liveMap);
    break;
    
    
  case 1: 
   // println("Infrared");  // Prints "One"
    // update the cam
  kinect.update();
image(kinect.irImage(), 0,-0);
server.sendImage(kinect.irImage());
    break;
}


  

  
  
  // DATA DISPLAY
  noStroke();
  color c = color(0, 150);
  fill(c);
  rect(90, 390, 330, 200);

  fill(255, 255, 255);
  textSize(12);
  text("front : " + front +  " cm" + "    " +  "back : " + back +  " cm" + "    " +  "fps: " + (int)frameRate, 100, 430);
  text("socket: 127.0.0.1:" + portIn + "/trap" + "   " + "outPort: " + portOut, 100, 446);
  text("y@fubbi.com", 100, 462);
  fill(255, 255, 255);
  textSize(12);
  text("[ISADORA CTRL] TRAPPING v1.0", 100, 415);
  
  
  


textFont(mono);
  
  
  fill(255, 0, 0);
  textSize(12);
  textAlign(RIGHT);
  if (mode == 0){
  text("TRAP MODE", 408, 415);
  }
  if (mode == 1){
  text("IR MODE", 408, 415);
  }
  textAlign(LEFT);
  

  

//  text("RCV: " + ping, 380, 415);
}



// listen for OSC
void oscEvent(OscMessage theOscMessage) {

  if(theOscMessage.isPlugged()==false) {
  /* print the address pattern and the typetag of the received OscMessage */
//    println("### received an osc message.");
//    println("### addrpattern\t"+theOscMessage.addrPattern());
//    println("### typetag\t"+theOscMessage.typetag());
  }
}
